// LevenshteinDistance.cpp : Definiert die exportierten Funktionen fr die DLL-Anwendung.
//

#include "stdafx.h"
#include "LevenshteinDistance.h"


#ifdef WIN32
/*
 * Number of micro-seconds between the beginning of the Windows epoch
 * (Jan. 1, 1601) and the Unix epoch (Jan. 1, 1970).
 *
 * This assumes all Win32 compilers have 64-bit support.
 */
#if defined(_MSC_VER) || defined(_MSC_EXTENSIONS) || defined(__WATCOMC__)
  #define DELTA_EPOCH_IN_USEC  11644473600000000Ui64
#else
  #define DELTA_EPOCH_IN_USEC  11644473600000000ULL
#endif
typedef	unsigned long long	u_int64_t;
static u_int64_t filetime_to_unix_epoch (const FILETIME *ft)
{
    u_int64_t res = (u_int64_t) ft->dwHighDateTime << 32;

    res |= ft->dwLowDateTime;
    res /= 10;                   /* from 100 nano-sec periods to usec */
    res -= DELTA_EPOCH_IN_USEC;  /* from Win epoch to Unix epoch */
    return (res);
}
#endif

BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
    switch (fdwReason)
    {
        case DLL_PROCESS_ATTACH:
            // attach to process
            // return FALSE to fail DLL load
            break;

        case DLL_PROCESS_DETACH:
            // detach from process
            break;

        case DLL_THREAD_ATTACH:
            // attach to thread
            break;

        case DLL_THREAD_DETACH:
            // detach from thread
            break;
    }
    return TRUE; // succesful
}


#ifdef __cplusplus
// Inhalt
// min3($a,$b,$c) - Minimum aus drei Werten wird zurckgegben
// LevenshteinDistance($s, $t) - Gibt Levenshteinabstand der beide Zeichenketten zurck
//   Fr die Levenshtein-Distanz gelten einige obere und untere Schranken:
//       * sie betrgt mindestens den Unterschied der Lngen beider Strings
//       * sie betrgt hchstens die Lnge des lngeren Strings
//       * sie ist dann und nur dann 0, wenn beide Strings identisch sind



int min3(int a, int b, int c) {
   int dummy;

   dummy = a;
   if (b < dummy) {dummy = b;}
   if (c < dummy) {dummy = c;}
   return dummy;
}


LEVENSHTEINDISTANCE_API int WINAPI LevenshteinDistance (TCHAR * S1, TCHAR * S2) {
    int n = lstrlen (S1), m = lstrlen (S2);
    int i, j, row, col, cost, result ;
    int *d ;
    HANDLE hHeap ;
 

    if (!lstrcmpi (S1, S2)) return 0;
    if (!n) return m;
    if (!m) return n;

    row = n + 1 ;
    col = m + 1 ;

    //int d[n+1][m+1], i, j, cost;
    hHeap = GetProcessHeap ( ) ;
    d = (int*)HeapAlloc (hHeap, HEAP_ZERO_MEMORY, row * col * sizeof(int)) ;

    for (i=0;i<=n;i++) d[i*col+0] = i;
    for (j=0;j<=m;j++) d[0*col+j] = j;

    for (i=1;i<=n;i++) {
        for (j=1;j<=m;j++) {
            if (S1[i-1] == S2[j-1])
                cost = 0;
            else
                cost = 1;

            d[i*col+j] = min3 (d[(i-1)*col+j] + 1, d[i*col+(j-1)] + 1, d[(i-1)*col+(j-1)] + cost);
        }
    }

    result = d[n*col+m] ;
    HeapFree (hHeap, 0, (LPVOID)d) ;
    return result ;
}


#endif // __cplusplus
